package demo.server_push.comet;

import java.util.Timer;
import java.util.TimerTask;

import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.Desktop;
import org.zkoss.zk.ui.DesktopUnavailableException;
import org.zkoss.zk.ui.Executions;
import org.zkoss.zk.ui.select.SelectorComposer;
import org.zkoss.zk.ui.select.annotation.Listen;
import org.zkoss.zk.ui.select.annotation.Wire;
import org.zkoss.zul.Image;
import org.zkoss.zul.Listbox;
import org.zkoss.zul.Messagebox;

public class CometServerPushController extends SelectorComposer<Component> {

	@Wire
	private Listbox maxDiameter;

	@Wire
	private Image info;
	
	private Timer timer;
	private InfoRenderer infoRenderer;
	
	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);
		
		infoRenderer = new InfoRenderer(200, 200);
	}

	@Listen("onClick = #start") 
	public void startWorker() {
		final Desktop desktop = Executions.getCurrent().getDesktop();
		if (desktop.isServerPushEnabled()) {
			Messagebox.show("Already started");
		} else {
			desktop.enableServerPush(true);
			if(timer != null)timer.cancel();
			timer = new Timer();
			timer.schedule(createUpdateTask(), 0, 1000);
		}
	}
	
	@Listen("onClick = #stop") 
	public void stopWorker() {
		final Desktop desktop = Executions.getCurrent().getDesktop();
		if (desktop.isServerPushEnabled()) {
			desktop.enableServerPush(false);
		} else {
			Messagebox.show("Already stopped");
		}
		timer.cancel();
	}

	private TimerTask createUpdateTask() {
		return new TimerTask() {
			@Override
			public void run() {
				updateInfo();
			}
		};
	}

	private void updateInfo() {
		try {
			Desktop desktop = info.getDesktop();
			if(desktop == null) {
				timer.cancel();
				return;
			}
			
			Executions.activate(desktop);
			try {
				int drawSize = Integer.parseInt((String)maxDiameter.getSelectedItem().getValue());
				((Image) info).setContent(infoRenderer.render(drawSize));
			} finally {
				Executions.deactivate(desktop);
			}
		} catch (DesktopUnavailableException ex) {
			System.out.println("Desktop currently unavailable");
			timer.cancel();
		} catch (InterruptedException e) {
			System.out.println("The server push thread interrupted");
			timer.cancel();
		}
	}
}
