package demo.server_push.comet;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Ellipse2D;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.util.ArrayList;
import java.util.List;

public class InfoRenderer {

	private static Color colors[] = { Color.blue, Color.cyan, Color.green,
			Color.magenta, Color.orange, Color.pink, Color.red, Color.yellow,
			Color.lightGray, Color.white };

	private Stroke lineStroke = new BasicStroke(1.5f);

	private int width;
	private int height;

	public InfoRenderer(int width, int height) {
		this.width = width;
		this.height = height;
	}

	public RenderedImage render(int drawSize) {
		List<Shape> shapes = prepareShapes(drawSize);
		
		BufferedImage bufferedImage = new BufferedImage(width, height,
				BufferedImage.TYPE_INT_RGB);

		Graphics2D g2d = createGraphics2D(bufferedImage);
		
		g2d.setStroke(lineStroke);
		for (Shape shape : shapes) {
			g2d.setColor(colors[(int)(Math.random() * colors.length)]);
			g2d.draw(shape);
		}
		
		g2d.dispose();
		return bufferedImage;
	}

	private List<Shape> prepareShapes(int drawSize) {
		int numShapes = 480 / drawSize;
		List<Shape> shapes = new ArrayList<Shape>(numShapes);
		for (int i = 0; i < numShapes; i++) {
			shapes.add(randomEllipse(drawSize));
		}
		return shapes;
	}

	private Shape randomEllipse(int drawSize) {
		float size = (float) (Math.random() * drawSize);
		float x = (float) (Math.random() * width) - size * 0.5f;
		float y = (float) (Math.random() * height) - size * 0.5f;
		return new Ellipse2D.Float(x, y, size, size);
	}

	private Graphics2D createGraphics2D(BufferedImage bufferedImage) {
		Graphics2D g2d = bufferedImage.createGraphics();
		g2d.setBackground(Color.BLACK);
		g2d.clearRect(0, 0, width, height);
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		return g2d;
	}
}
